#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Data::Path::Parser class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
# Comments:
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 21;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Data::Path::Parser';

        use_ok('Tester');
        use_ok($TESTED_CLASS);
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
diag("$TESTED_CLASS version: " . $Template::Module::VERSION) if Tester::D();

# Now attempt to create a class instance and check it
my $instance = $TESTED_CLASS->new();
isa_ok($instance, $TESTED_CLASS);

# View the created instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D();

# Verify the instance is empty
is(Dumper($instance),
'$VAR1 = bless( {}, \'Data::Path::Parser\' );
',
"Instance print test 01");

# Test Mentat message path
my $message_path = 'Alert/Target[1]/Node/Address/@category';

# Expected result
my $expected_result = [
        [
            'Alert',
            'elem',
            undef,
            'Target[1]/Node/Address/@category',
        ],
        [
            'Target',
            'elem',
            1,
            'Node/Address/@category',
        ],
        [
            'Node',
            'elem',
            undef,
            'Address/@category',
        ],
        [
            'Address',
            'elem',
            undef,
            '@category',
        ],
        [
            'category',
            'attr',
            undef,
            undef,
        ]
    ];


# Result storage variables
my ( $name, $type, $index, $rest, $xpath);

# Check the error handling
eval {
    ($name, $type, $index, $rest) = Data::Path::Parser->parse_next_chunk('');
};
if($@) { like($@, qr/Data::Path::Parser::parse_next_chunk: undefined next data path node/, "Error test 02"); } else { fail("Error test 02"); }
eval {
    ($name, $type, $index, $rest) = Data::Path::Parser->parse_next_chunk('Alert9]/Target/Node/Address/@category');
};
if($@) { like($@, qr/Data::Path::Parser::parse_next_chunk: invalid Mentat message node: 'Alert9]'/, "Error test 03"); } else { fail("Error test 03"); }
eval {
    ($name, $type, $index, $rest) = Data::Path::Parser->parse_next_chunk('Target[0]/Node/Address/@category');
};
if($@) { like($@, qr/Data::Path::Parser::parse_next_chunk: invalid index value '0', index starts at '1'/, "Error test 03"); } else { fail("Error test 04"); }
eval {
    ($name, $type, $index, $rest) = Data::Path::Parser->parse_next_chunk('@category[2]');
};
if($@) { like($@, qr/Data::Path::Parser::parse_next_chunk: atribute node '\@category\[2\]' must have only one instance/, "Error test 04"); } else { fail("Error test 05"); }
eval {
    ($name, $type, $index, $rest) = Data::Path::Parser->parse_next_chunk('@category/test');
};
if($@) { like($@, qr/Data::Path::Parser::parse_next_chunk: atribute node '\@category' may not contain any child nodes/, "Error test 05"); } else { fail("Error test 06"); }

# Parse the data path chunk by chunk
my @result;
$rest = $message_path;
do {
    ($name, $type, $index, $rest) = Data::Path::Parser->parse_next_chunk($rest);
    push(@result, [$name, $type, $index, $rest]);
} while ($rest);
is_deeply(\@result, $expected_result, 'Parse test 01');

# Parse the whole Mentat message path in one step
my @chunks = Data::Path::Parser->parse($message_path);
is_deeply(\@chunks, $expected_result, 'Parse test 02');

# Check the catching variant of parse method
my @chunks2 = Data::Path::Parser->parse_quiet($message_path);
is_deeply(\@chunks2, $expected_result, 'Parse test 03');
is(Data::Path::Parser->parse_quiet('Alert9]/Target/Node/Address/@category'), undef, 'Parse test 04');

# Check the join
is(Data::Path::Parser->join('Alert/Target/Node/Address', '@category'), 'Alert/Target/Node/Address/@category', 'Join test 01');
is(Data::Path::Parser->join('Alert/Target', 'Node', 2), 'Alert/Target/Node[2]', 'Join test 01');

# Check the validation
is(Data::Path::Parser->validate($message_path), 1, 'Validation test 01');
is(Data::Path::Parser->validate(''), 0, 'Validation test 02');
is(Data::Path::Parser->validate('Alert9]/Target/Node/Address/@category'), 0, 'Validation test 03');
is(Data::Path::Parser->validate('Alert/Target/Node/Address/@category[1]'), 0, 'Validation test 04');
is(Data::Path::Parser->validate('Alert/Target/Node/Address/@category/test'), 0, 'Validation test 05');
is(Data::Path::Parser->validate('Alert/Target[0]/Node/Address/@category'), 0, 'Validation test 06');
